/*
* This file is part of project OpenEmbroidery. It's copyrighted by
* the contributors recorded in the version control history of the file.
* Original project location https://code.electrolab.fr/openEmbroidery/openEmbroidery_software
*
* SPDX-License-Identifier: CECILL-2.1
* License-Filename: Licence_CeCILL_V2.1-en.txt
*/

#pragma once

#include "curves/OE_curve.h"
#include "stitchs/OE_stitchs.h"
#include "curves/OE_joincurve.h"

namespace Pakal { class  Archive; }

class OE_linestitch : public OE_stitchs
{
	public:
		OE_linestitch();
		//OE_linestitch(OE_thread* thread, float len, float width, OE_pattern* pattern);
		//OE_linestitch(OE_thread* thread, float len, float width, OE_pattern* pattern, float offset);
		OE_linestitch(OE_thread* thread, float len, float width, OE_pattern* pattern, unsigned patternSteps = 1, float xOffset = 0, float yOffset = 0);

		~OE_linestitch();

		virtual void persist(Pakal::Archive* archive) override;

		virtual bool check();

		//virtual void refresh();

		virtual void refresh(float dpi, bool force = false);

		void setLen(float len);
		void setWidth(float width);
		void setPattern(OE_pattern* stitch);
		void setXOffset(float offset);
		void setYOffset(float offset);
		void setReverse(bool reverse);

		OE_joincurve* getJoincurve();
		float getLen();
		float getWidth();
		OE_pattern* getPattern();
		float getXOffset();
		float getYOffset();
		bool getReverse();

		static bool initPatterns();

		void delDependency(OE_base * object);
		virtual void refreshDependency();

	protected:
		OE_joincurve* curve;
		bool reverse;
		float len;
		float width;
		OE_pattern* pattern;
		unsigned patternSteps;
		float xOffset;
		float yOffset;
};

