/*
* This file is part of project OpenEmbroidery. It's copyrighted by
* the contributors recorded in the version control history of the file.
* Original project location https://code.electrolab.fr/openEmbroidery/openEmbroidery_software
*
* SPDX-License-Identifier: CECILL-2.1
* License-Filename: Licence_CeCILL_V2.1-en.txt
*/

#include "OE_io.h"
#include "OE_document.h"
#include "stitchs/OE_birailstitch.h"
#include "stitchs/OE_linestitch.h"
#include "stitchs/OE_linkstitch.h"
#include "stitchs/OE_fillstitch.h"
#include "stitchs/OE_staticstitch.h"
#include "OE_utils.h"
#include <iostream>
#include <fstream>

#include "Archive.h"
#include "JsonWriter.h"
#include "JsonReader.h"

static bool loadFromOE(std::string path, OE_document* document);
static bool saveToOE(const OE_document* document, std::string path);

extern const OE_io ioOe;
const OE_io ioOe = {"oe",
                    "OpenEmbroidery",
                    loadFromOE,
                    saveToOE};

static bool loadFromOE(std::string path, OE_document* document)
{
	OE_document::ScopeLock lock(*document);
	OE_ifstream in(path);
	if (!in.is_open())
	{
		return false;
	}
	Pakal::SimpleFactoyManager manager;
	manager.declare_object<OE_birailstitch>("OE_birailstitch", []() { return new OE_birailstitch(nullptr); });
	manager.declare_object<OE_linestitch>("OE_linestitch", []() { return new OE_linestitch(); });
	manager.declare_object<OE_linkstitch>("OE_linkstitch", []() { return new OE_linkstitch(); });
	manager.declare_object<OE_fillstitch>("OE_fillstitch", []() { return new OE_fillstitch(); });
	manager.declare_object<OE_staticstitch>("OE_staticstitch", []() { return new OE_staticstitch(); });
	Pakal::JsonReader json_reader(&manager);
	json_reader.read(in, "document", *document);
	in.close();
	for (auto& stitch : document->stitchs)
	{
		stitch->refreshDependency();
	}
	return true;
}


static bool saveToOE(const OE_document* document, std::string path)
{
	OE_ofstream out(path);
	if (!out.is_open())
	{
		return false;
	}
	Pakal::JsonWriter json_writer(true);
	json_writer.write(out, "document", *(OE_document*)document); // drop constness due to pakal structure
	out.close();
	return true;
}










